@@ -9,6 +9,7 @@ from collections import Counter, defaultdict
 from omspy.base import *
 from copy import deepcopy
 from sqlite_utils import Database
+from omspy.models import OrderLock
 
 
 def get_option(spot: float, num: int = 0, step: float = 100.0) -> float:
@@ -66,6 +67,14 @@ def create_db(dbname: str = ":memory:") -> Union[Database, None]:
 
 
 class Order(BaseModel):
+    """
+    The basic Order Class
+    _attrs
+        attributes to update when data received from broker
+    _frozen_attrs
+        attributes frozen; cannot be changed when modifying orders
+    """
+
     symbol: str
     side: str
     quantity: int = 1
@@ -87,7 +96,7 @@ class Order(BaseModel):
     validity: str = "DAY"
     status: Optional[str] = None
     expires_in: int = 0
-    timezone: str = "UTC"
+    timezone: str = "local"
     client_id: Optional[str] = None
     convert_to_market_after_expiry: bool = False
     cancel_after_expiry: bool = True
@@ -115,6 +124,8 @@ class Order(BaseModel):
         "average_price",
     )
     _exclude_fields: Set[str] = {"connection"}
+    _lock: Optional[OrderLock] = None
+    _frozen_attrs: Set[str] = {"symbol", "side"}
 
     class Config:
         underscore_attrs_are_private = True
@@ -136,6 +147,8 @@ class Order(BaseModel):
             ).seconds
         else:
             self.expires_in = abs(self.expires_in)
+        if self._lock is None:
+            self._lock = OrderLock()
 
     @validator("quantity", always=True, allow_reuse=True)
     def quantity_not_negative(cls, v):
@@ -166,6 +179,18 @@ class Order(BaseModel):
         else:
             return False
 
+    @property
+    def is_done(self) -> bool:
+        """
+        returns True if the order is either COMPLETE or CANCELED or REJECTED else False
+        """
+        if self.is_complete:
+            return True
+        elif self.status in ("CANCELLED", "CANCELED", "REJECTED"):
+            return True
+        else:
+            return False
+
     @property
     def time_to_expiry(self) -> int:
         now = pendulum.now(tz=self.timezone)
@@ -186,6 +211,43 @@ class Order(BaseModel):
     def has_parent(self) -> bool:
         return True if self.parent_id else False
 
+    @property
+    def lock(self) -> Union[OrderLock, None]:
+        return self._lock
+
+    def _get_other_args_from_attribs(
+        self, broker: Any, attribute: str, attribs_to_copy: Optional[Set] = None
+    ) -> Dict[str, str]:
+        """
+        Get other arguments for the order from attributes
+        broker
+            valid broker instance
+        attribute
+            attribute to search for in broker
+        attribs_to_copy
+            extra attributes to be copied
+        Note
+        ----
+        1) The broker instance is first searched for the valid attribute and it is overriden with attribs_to_copy
+        """
+        if attribs_to_copy is None:
+            attribs_to_copy = set()
+        else:
+            # Convert any iterable
+            attribs_to_copy = set([x for x in attribs_to_copy])
+        if hasattr(broker, attribute):
+            attribs = getattr(broker, attribute)
+            for attrib in attribs:
+                attribs_to_copy.add(attrib)
+        other_args = dict()
+        if attribs_to_copy:
+            for key in attribs_to_copy:
+                if hasattr(self, key):
+                    value = getattr(self, key)
+                    if value:
+                        other_args[key] = value
+        return other_args
+
     def update(self, data: Dict[str, Any], save: bool = True) -> bool:
         """
         Update order based on information received from broker
@@ -195,21 +257,26 @@ class Order(BaseModel):
         Note
         ----
         1) Information is updated only for those keys specified in attrs
-        2) Information is updated only when the order is not completed
+        2) Information is updated only when the order is still pending; completed/rejected/canceled orders not updated
+        3) Update pending quantity if it is not in data
         """
-        if not (self.is_complete):
+        if not (self.is_done):
             for att in self._attrs:
                 val = data.get(att)
                 if val:
                     setattr(self, att, val)
             self.last_updated_at = pendulum.now(tz=self.timezone)
+            if not ("pending_quantity" in data):
+                self.pending_quantity = self.quantity - self.filled_quantity
             if self.connection and save:
                 self.save_to_db()
             return True
         else:
             return False
 
-    def execute(self, broker: Any, **kwargs) -> Optional[str]:
+    def execute(
+        self, broker: Any, attribs_to_copy: Optional[Set] = None, **kwargs
+    ) -> Optional[str]:
         """
         Execute an order on a broker, place a new order
         kwargs
@@ -219,8 +286,12 @@ class Order(BaseModel):
         Only new arguments added to the order in keyword arguments
         """
         # Do not place a new order if this order is complete or has order_id
+
         from omspy.base import Broker as base_broker
 
+        other_args = self._get_other_args_from_attribs(
+            broker, attribute="attribs_to_copy_execute", attribs_to_copy=attribs_to_copy
+        )
         if not (self.is_complete) and not (self.order_id):
             order_args = {
                 "symbol": self.symbol.upper(),
@@ -232,6 +303,7 @@ class Order(BaseModel):
                 "disclosed_quantity": self.disclosed_quantity,
             }
             dct = {k: v for k, v in kwargs.items() if k not in order_args.keys()}
+            order_args.update(other_args)
             order_args.update(dct)
             order_id = broker.order_place(**order_args)
             self.order_id = order_id
@@ -241,13 +313,46 @@ class Order(BaseModel):
         else:
             return self.order_id
 
-    def modify(self, broker: Any, **kwargs):
+    def modify(
+        self, broker: Any, attribs_to_copy: Optional[Tuple] = None, **kwargs
+    ) -> None:
         """
         Modify an existing order
+        broker
+            broker to execute the order
+        attribs
+            attributes to be copied from the order
+        kwargs
+            keyword arguments for modification
+        Note
+        ----
+        1)resolution for order args - default arguments are created for modify, attribs_to_copy are added next and finally kwargs are updated. If the same attribute is found in all three, the user provided kwargs wins
         """
+        if not (self.lock.can_modify):
+            logging.debug(
+                f"Order not modified since lock is modified till {self.lock.modification_lock_till}"
+            )
+            return
+        other_args = self._get_other_args_from_attribs(
+            broker, attribute="attribs_to_copy_modify", attribs_to_copy=attribs_to_copy
+        )
+        args_to_add = dict()
+        keys = [
+            "order_id",
+            "quantity",
+            "price",
+            "trigger_price",
+            "order_type",
+            "disclosed_quantity",
+        ]
         for k, v in kwargs.items():
-            if hasattr(self, k):
-                setattr(self, k, v)
+            if k not in self._frozen_attrs:
+                if hasattr(self, k):
+                    setattr(self, k, v)
+                    if k not in keys:
+                        args_to_add[k] = v
+                else:
+                    other_args[k] = v
         order_args = {
             "order_id": self.order_id,
             "quantity": self.quantity,
@@ -256,17 +361,28 @@ class Order(BaseModel):
             "order_type": self.order_type.upper(),
             "disclosed_quantity": self.disclosed_quantity,
         }
+        order_args.update(other_args)
+        order_args.update(args_to_add)
         if self._num_modifications < self.max_modifications:
             broker.order_modify(**order_args)
             self._num_modifications += 1
         else:
             logging.info(f"Maximum number of modifications exceeded")
 
-    def cancel(self, broker: Any):
+    def cancel(self, broker: Any, attribs_to_copy: Optional[Set] = None) -> None:
         """
         Cancel an existing order
         """
-        broker.order_cancel(order_id=self.order_id)
+        if not (self.lock.can_cancel):
+            logging.debug(
+                f"Order not canceled since lock is modified till {self.lock.cancellation_lock_till}"
+            )
+            return
+        other_args = self._get_other_args_from_attribs(
+            broker, attribute="attribs_to_copy_cancel", attribs_to_copy=attribs_to_copy
+        )
+        if self.order_id is not None:
+            broker.order_cancel(order_id=self.order_id, **other_args)
 
     def save_to_db(self) -> bool:
         """
@@ -292,6 +408,22 @@ class Order(BaseModel):
         order = Order(**dct)
         return order
 
+    def add_lock(self, code: int, seconds: float):
+        """
+        Create a lock on modify or cancel function
+        code
+            1 to lock modify and 2 to lock cancel
+        seconds
+            duration to lock the function
+        Note
+        ----
+        1) Lock is created only on modify or cancel
+        """
+        if code == 1:
+            self.lock.modify(seconds=seconds)
+        elif code == 2:
+            self.lock.cancel(seconds=seconds)
+
 
 class CompoundOrder(BaseModel):
     broker: Any
@@ -475,7 +607,7 @@ class CompoundOrder(BaseModel):
             order_args.update(kwargs)
             order.execute(broker=self.broker, **order_args)
 
-    def check_flags(self):
+    def check_flags(self) -> None:
         """
         Check for flags on each order and take suitable action
         """
@@ -502,11 +634,13 @@ class CompoundOrder(BaseModel):
         order.parent_id = self.id
         if not (order.connection):
             order.connection = self.connection
+        if not (order.id):
+            order.id = uuid.uuid4().hex
         order.save_to_db()
         self.orders.append(order)
         return order.id
 
-    def save(self):
+    def save(self) -> None:
         """
         Save all orders to database
         """
@@ -514,3 +648,84 @@ class CompoundOrder(BaseModel):
             for order in self.orders:
                 order.save_to_db()
         pass
+
+
+class OrderStrategy(BaseModel):
+    """
+    An order strategy is a list of strategies that
+    is made up of a list of compound orders
+    """
+
+    broker: Any
+    id: Optional[str] = None
+    ltp: defaultdict = Field(default_factory=defaultdict)
+    orders: List[CompoundOrder] = Field(default_factory=list)
+
+    class Config:
+        underscore_attrs_are_private = True
+        arbitrary_types_allowed = True
+
+    def __init__(self, **data) -> None:
+        super().__init__(**data)
+        if not (self.id):
+            self.id = uuid.uuid4().hex
+
+    @property
+    def positions(self) -> Counter:
+        c: Counter = Counter()
+        for order in self.orders:
+            pos = order.positions
+            c.update(pos)
+        return c
+
+    def update_ltp(self, last_price: Dict[str, float]):
+        for symbol, ltp in last_price.items():
+            self.ltp[symbol] = ltp
+        for order in self.orders:
+            order.update_ltp(last_price)
+        return self.ltp
+
+    def update_orders(self, data: Dict[str, Dict[str, Any]]) -> None:
+        """
+        Update all orders
+        data
+            data as dictionary with key as broker order_id
+        """
+        for order in self.orders:
+            order.update_orders(data)
+
+    @property
+    def mtm(self) -> Counter:
+        c: Counter = Counter()
+        for order in self.orders:
+            mtm = order.mtm
+            c.update(mtm)
+        return c
+
+    def run(self, ltp: Dict[str, float]) -> None:
+        """
+        Run all orders with the given data
+        ltp
+            last price data as a dictionary
+        """
+        for order in self.orders:
+            if hasattr(order, "run"):
+                if callable(getattr(order, "run")):
+                    order.run(ltp)
+
+    def add(self, order: CompoundOrder) -> None:
+        """
+        Add a compound order to the existing strategy
+        """
+        self.orders.append(order)
+
+    def save(self) -> None:
+        """
+        Save all orders to database
+        Note
+        ----
+        1) Orders are saved based on the preferences of each compound order; so this doesn't save everything
+        """
+        for order in self.orders:
+            order.save()
+        pass
